@@ -51,6 +51,7 @@ iBool   isSpace_Char        (iChar);
 iBool   isAlpha_Char        (iChar);
 iBool   isNumeric_Char      (iChar);
 iBool   isAlphaNumeric_Char (iChar);
+iBool   isPunct_Char        (iChar);
 
 iLocalDef iBool isVariationSelector_Char(iChar c) {
     return (c >= 0xfe00 && c <= 0xfe0f) || (c >= 0xe0100 && c <= 0xe0121);
@@ -112,8 +113,10 @@ iString *       newBlock_String     (const iBlock *utf8Data);
 iString *       newFormat_String    (const char *format, ...);
 iString *       copy_String         (const iString *);
 
-iLocalDef iString *newRange_String  (const iRangecc range) { return newCStrN_String(range.start, size_Range(&range)); }
-iLocalDef iString *newLocal_String  (const iBlock *localChars) { return newLocalCStrN_String(cstr_Block(localChars), size_Block(localChars)); }
+iLocalDef iString * newUtf32_String     (const iChar *ucs) { return newUnicode_String(ucs); }
+iLocalDef iString * newUtf32N_String    (const iChar *ucs, size_t n) { return newUnicodeN_String(ucs, n); }
+iLocalDef iString * newRange_String     (const iRangecc range) { return newCStrN_String(range.start, size_Range(&range)); }
+iLocalDef iString * newLocal_String     (const iBlock *localChars) { return newLocalCStrN_String(cstr_Block(localChars), size_Block(localChars)); }
 
 iString *           collectNewFormat_String (const char *format, ...);
 iLocalDef iString * collectNewCStr_String   (const char *cstr) { return collect_String(newCStr_String(cstr)); }
@@ -141,14 +144,20 @@ iString *       mid_String          (const iString *, size_t charStartPos, size_
 iString *       upper_String        (const iString *);
 iString *       lower_String        (const iString *);
 iStringList *   split_String        (const iString *, const char *separator);
-iString *       urlEncode_String    (const iString *);
-iString *       urlEncodeExclude_String(const iString *d, const char *excluded);
-iString *       urlDecode_String    (const iString *);
-iString *       urlDecodeExclude_String(const iString *, const char *excluded);
 iChar           first_String        (const iString *);
 iChar           last_String         (const iString *);
 iBlock *        toLocal_String      (const iString *);
 iBlock *        toUtf16_String      (const iString *);
+iBlock *        toUnicode_String    (const iString *);
+
+iLocalDef iBlock *toUtf32_String    (const iString *d) { return toUnicode_String(d); }
+
+iString *       urlEncode_String                (const iString *);
+iString *       urlDecode_String                (const iString *);
+iString *       urlEncodeExclude_String         (const iString *, const char *excluded);
+iString *       urlDecodeExclude_String         (const iString *, const char *excluded);
+iString *       maybeUrlEncodeExclude_String    (const iString *, const char *excluded); /* may return NULL */
+iString *       maybeUrlDecodeExclude_String    (const iString *, const char *excluded); /* may return NULL */
 
 /**
  * Returns a pointer to the string converted to the current locale's encoding.
@@ -268,12 +277,10 @@ iLocalDef iRangecc range_CStr(const char *cstr) {
     return rangeN_CStr(cstr, strlen(cstr));
 }
 
-const char *    cstr_Rangecc        (iRangecc range); /* returns NULL-terminated collected copy */
-const iString * string_Rangecc      (iRangecc range); /* returns a collected String */
+const char *    cstr_Rangecc        (iRangecc); /* returns NULL-terminated collected copy */
+const iString * string_Rangecc      (iRangecc); /* returns a collected String */
 
-iLocalDef iBool isNull_Rangecc(const iRangecc d) {
-    return d.start == NULL;
-}
+size_t          length_Rangecc      (iRangecc); /* returns number of characters in the range */
 
 #define         cmp_Rangecc(d, cstr) cmpCStrSc_Rangecc((d), (cstr), &iCaseSensitive)
 int             cmpCStrSc_Rangecc   (iRangecc, const char *cstr, const iStringComparison *);
@@ -281,6 +288,9 @@ int             cmpCStrNSc_Rangecc  (iRangecc, const char *cstr, size_t n, const
 iBool           startsWithSc_Rangecc(iRangecc, const char *cstr, const iStringComparison *);
 iBool           endsWithSc_Rangecc  (iRangecc, const char *cstr, const iStringComparison *);
 
+iLocalDef iBool isNull_Rangecc(const iRangecc d) {
+    return d.start == NULL;
+}
 iLocalDef iBool equal_Rangecc(const iRangecc d, const char *cstr) {
     return cmp_Rangecc(d, cstr) == 0;
 }
