@@ -44,19 +44,67 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.</small>
 
 iDeclareType(Context)
 
-#define DEFAULT_BUF_SIZE 1024
+#define DEFAULT_BUF_SIZE 8192
 
 static iContext *context_;
 static iBool isPrngSeeded_;
 
 static void initContext_(void);
+static iTlsCertificate *newX509Chain_TlsCertificate_(X509 *cert, STACK_OF(X509) *chain);
+static void certificateVerifyFailed_TlsRequest_(iTlsRequest *, const iTlsCertificate *cert);
 
 struct Impl_Context {
-    SSL_CTX *ctx;
-    X509_STORE *certStore;
+    SSL_CTX *             ctx;
+    X509_STORE *          certStore;
+    iTlsRequestVerifyFunc userVerifyFunc;
+    tss_t                 tssKeyCurrentRequest;
 };
 
+static iTlsRequest *currentRequestForThread_Context_(iContext *d) {
+    return tss_get(context_->tssKeyCurrentRequest);
+}
+
+static void setCurrentRequestForThread_Context_(iContext *d, iTlsRequest *request) {
+    tss_set(context_->tssKeyCurrentRequest, request);    
+}
+
+static int verifyCallback_Context_(int preverifyOk, X509_STORE_CTX *storeCtx) {
+    if (preverifyOk) {
+        return 1; /* OpenSSL says it's OK */
+    }
+    iAssert(context_); /* must've been initialized by now */
+    iContext *d     = context_;
+    const int depth = X509_STORE_CTX_get_error_depth(storeCtx);
+    X509 *    x509  = X509_STORE_CTX_get_current_cert(storeCtx);
+    X509_up_ref(x509); /* keep a reference */
+    iTlsCertificate *cert = newX509Chain_TlsCertificate_(x509, NULL);
+#if 0
+    /* Debugging. */ {
+        X509_NAME *name = X509_get_subject_name(x509);
+        printf("[TlsRequest] verifyCallback_Context: depth=%d\n", depth);
+        X509_NAME_print_ex_fp(stdout, name, 0, 0);
+        printf("\n");
+        iBlock *fp = publicKeyFingerprint_TlsCertificate(cert);
+        printf("key fp: %s\n", cstrCollect_String(hexEncode_Block(fp)));
+        delete_Block(fp);
+    }
+#endif
+    int result = 1; /* accept everything by default */
+    if (d->userVerifyFunc) {
+        iTlsRequest *request = currentRequestForThread_Context_(d);
+        iAssert(request != NULL);
+        result = d->userVerifyFunc(request, cert, depth) ? 1 : 0;
+        if (!result) {
+            certificateVerifyFailed_TlsRequest_(request, cert);
+        }
+    } 
+    delete_TlsCertificate(cert); /* free the reference */
+    return result;
+}
+
 void init_Context(iContext *d) {
+    d->tssKeyCurrentRequest = 0;
+    tss_create(&d->tssKeyCurrentRequest, NULL);
 #if OPENSSL_API_COMPAT >= 0x10100000L
     OPENSSL_init_ssl(OPENSSL_INIT_LOAD_SSL_STRINGS | OPENSSL_INIT_LOAD_CRYPTO_STRINGS, NULL);
     OPENSSL_init_crypto(OPENSSL_INIT_ADD_ALL_CIPHERS | OPENSSL_INIT_ADD_ALL_DIGESTS, NULL);
@@ -73,7 +121,8 @@ void init_Context(iContext *d) {
         iDebug("[TlsRequest] Failed to initialize OpenSSL\n");
         iAssert(d->ctx);
     }
-    SSL_CTX_set_verify(d->ctx, SSL_VERIFY_NONE, NULL); /* allow manual verification */
+    d->userVerifyFunc = NULL;
+    SSL_CTX_set_verify(d->ctx, SSL_VERIFY_PEER, verifyCallback_Context_);
     /* Bug workarounds: https://www.openssl.org/docs/manmaster/man3/SSL_CTX_set_options.html */
     SSL_CTX_set_options(d->ctx, SSL_OP_ALL);
     d->certStore = NULL;
@@ -81,6 +130,7 @@ void init_Context(iContext *d) {
 
 void deinit_Context(iContext *d) {
     SSL_CTX_free(d->ctx);
+    tss_delete(d->tssKeyCurrentRequest);
 }
 
 iBool isValid_Context(iContext *d) {
@@ -101,6 +151,12 @@ void setCACertificates_TlsRequest(const iString *caFile, const iString *caPath)
     }
 }
 
+void setVerifyFunc_TlsRequest(iTlsRequestVerifyFunc verifyFunc) {
+    initContext_();
+    iContext *d = context_;
+    d->userVerifyFunc = verifyFunc;
+}
+
 iDefineTypeConstruction(Context)
 
 static void globalCleanup_TlsRequest_(void) {
@@ -159,13 +215,9 @@ void deinit_TlsCertificate(iTlsCertificate *d) {
     }
 }
 
-//static iTlsCertificate *newX509_TlsCertificate_(X509 *cert) {
-//    iTlsCertificate *d = new_TlsCertificate();
-//    d->cert = cert;
-//    return d;
-//}
-
 static iTlsCertificate *newX509Chain_TlsCertificate_(X509 *cert, STACK_OF(X509) *chain) {
+    /* Note that `chain` includes `cert`; a bit redundant. However, the chain is only
+       used when verifying the `cert` via OpenSSL's API. */
     iTlsCertificate *d = new_TlsCertificate();
     d->cert  = cert;
     d->chain = chain;
@@ -189,7 +241,7 @@ iTlsCertificate *newPemKey_TlsCertificate(const iString *certPem, const iString
 }
 
 static const iString *findName_(const iTlsCertificateName *names, enum iTlsCertificateNameType type) {
-    for (; names->text && names->type; names++) {
+    for (; names->type; names++) {
         if (names->type == type) {
             return names->text;
         }
@@ -284,6 +336,20 @@ iTlsCertificate *newSelfSignedRSA_TlsCertificate(
     return d;
 }
 
+iTlsCertificate *copy_TlsCertificate(const iTlsCertificate *d) {
+    iTlsCertificate *copy = new_TlsCertificate();
+    if (d->cert) {
+        X509_up_ref(d->cert);
+        copy->cert = d->cert;
+    }
+    copy->chain = d->chain ? sk_X509_dup(d->chain) : NULL;
+    if (d->pkey) {
+        EVP_PKEY_up_ref(d->pkey);
+        copy->pkey = d->pkey;
+    }
+    return copy;
+}
+
 iBool isEmpty_TlsCertificate(const iTlsCertificate *d) {
     return d->cert == NULL;
 }
@@ -406,19 +472,39 @@ iBool equal_TlsCertificate(const iTlsCertificate *d, const iTlsCertificate *othe
     return X509_cmp(d->cert, other->cert) == 0;
 }
 
+static void calcSHA256_(BIO *src, iBlock *dst) {
+    iBlock der;
+    init_Block(&der, 0);
+    readAllFromBIO_(src, &der);
+    SHA256(constData_Block(&der), size_Block(&der), data_Block(dst));
+    deinit_Block(&der);
+}
+
 iBlock *fingerprint_TlsCertificate(const iTlsCertificate *d) {
     iBlock *sha = new_Block(SHA256_DIGEST_LENGTH);
     if (d->cert) {
-        iBlock der;
-        init_Block(&der, 0);
-        /* Get the DER serialization of the certificate. */ {
-            BIO *buf = BIO_new(BIO_s_mem());
-            i2d_X509_bio(buf, d->cert);
-            readAllFromBIO_(buf, &der);
-            BIO_free(buf);
-        }
-        SHA256(constData_Block(&der), size_Block(&der), data_Block(sha));
-        deinit_Block(&der);
+        /* Get the DER serialization of the certificate. */
+        BIO *buf = BIO_new(BIO_s_mem());
+        i2d_X509_bio(buf, d->cert);
+        calcSHA256_(buf, sha);
+        BIO_free(buf);
+    }
+    return sha;
+}
+
+iBlock *publicKeyFingerprint_TlsCertificate(const iTlsCertificate *d) {
+    iBlock *sha = new_Block(SHA256_DIGEST_LENGTH);
+    if (!d->cert) {
+        return sha;
+    }
+    EVP_PKEY *pub = X509_get_pubkey(d->cert);
+    if (pub) {
+        /* Get the DER serialization of the public key. */
+        BIO *buf = BIO_new(BIO_s_mem());
+        i2d_PUBKEY_bio(buf, pub);
+        calcSHA256_(buf, sha);
+        BIO_free(buf);
+        EVP_PKEY_free(pub);
     }
     return sha;
 }
@@ -426,16 +512,11 @@ iBlock *fingerprint_TlsCertificate(const iTlsCertificate *d) {
 iBlock *privateKeyFingerprint_TlsCertificate(const iTlsCertificate *d) {
     iBlock *sha = new_Block(SHA256_DIGEST_LENGTH);
     if (d->pkey) {
-        iBlock der;
-        init_Block(&der, 0);
-        /* Get the DER serialization of the private key. */ {
-            BIO *buf = BIO_new(BIO_s_mem());
-            i2d_PrivateKey_bio(buf, d->pkey);
-            readAllFromBIO_(buf, &der);
-            BIO_free(buf);
-        }
-        SHA256(constData_Block(&der), size_Block(&der), data_Block(sha));
-        deinit_Block(&der);
+        /* Get the DER serialization of the private key. */
+        BIO *buf = BIO_new(BIO_s_mem());
+        i2d_PrivateKey_bio(buf, d->pkey);
+        calcSHA256_(buf, sha);
+        BIO_free(buf);
     }
     return sha;
 }
@@ -476,16 +557,20 @@ struct Impl_TlsRequest {
     iBlock           content;
     iBuffer *        result;
     iTlsCertificate *cert; /* server certificate */
+    iBool            certVerifyFailed;
     /* Internal state. */
     volatile enum iTlsRequestStatus status;
     iString *        errorMsg;
     iThread *        thread;
     iBool            notifyReady;
+    size_t           totalBytesToSend;
+    size_t           totalBytesSent;
     iBlock  *        incoming;
     iCondition       gotIncoming;
     iMutex           incomingMtx;
     iCondition       requestDone;
     iAudience *      readyRead;
+    iAudience *      sent;
     iAudience *      finished;
     /* OpenSSL state. */
     SSL *            ssl;
@@ -496,6 +581,7 @@ struct Impl_TlsRequest {
 
 iDefineObjectConstruction(TlsRequest)
 iDefineAudienceGetter(TlsRequest, readyRead)
+iDefineAudienceGetter(TlsRequest, sent)
 iDefineAudienceGetter(TlsRequest, finished)
 
 static void setError_TlsRequest_(iTlsRequest *d, const char *msg);
@@ -543,6 +629,7 @@ static void flushToSocket_TlsRequest_(iTlsRequest *d) {
     do {
         n = BIO_read(d->wbio, buf, sizeof(buf));
         if (n > 0) {
+            d->totalBytesToSend += n;
             writeData_Socket(d->socket, buf, n);
         }
         else if (!BIO_should_retry(d->wbio)) {
@@ -605,11 +692,14 @@ void init_TlsRequest(iTlsRequest *d) {
     d->status = initialized_TlsRequestStatus;
     d->thread = NULL;
     d->notifyReady = iFalse;
+    d->totalBytesToSend = 0;
+    d->totalBytesSent = 0;
     d->incoming = new_Block(0);
     init_Mutex(&d->incomingMtx);
     init_Condition(&d->gotIncoming);
     init_Condition(&d->requestDone);
     d->readyRead = NULL;
+    d->sent = NULL;
     d->finished = NULL;
     d->ssl = SSL_new(context_->ctx);
     /* We could also try BIO_s_socket() but all BSD socket related code should be encapsulated
@@ -635,6 +725,7 @@ void deinit_TlsRequest(iTlsRequest *d) {
     deinit_Mutex(&d->incomingMtx);
     delete_Block(d->incoming);
     delete_Audience(d->finished);
+    delete_Audience(d->sent);
     delete_Audience(d->readyRead);
     delete_String(d->errorMsg);
     delete_TlsCertificate(d->cert);
@@ -751,9 +842,11 @@ static iBool readIncoming_TlsRequest_(iTlsRequest *d) {
 }
 
 static iThreadResult run_TlsRequest_(iThread *thread) {
-    /* TODO: This thread seems unnecessary. The Socket already runs a thread -- we could execute
-       this via its observers, and block to encrypt outgoing data. */
     iTlsRequest *d = userData_Thread(thread);
+    /* Thread-local pointer to the current request so it can be accessed in the 
+       verify callback. */
+    iDebug("[TlsRequest] run_TlsRequest_: %zu bytes to send\n", size_Block(&d->sending));
+    setCurrentRequestForThread_Context_(context_, d);
     doHandshake_TlsRequest_(d);
     for (;;) {
         encrypt_TlsRequest_(d);
@@ -795,6 +888,11 @@ static void disconnected_TlsRequest_(iTlsRequest *d, iSocket *sock) {
     setStatus_TlsRequest_(d, finished_TlsRequestStatus);
 }
 
+static void bytesWritten_TlsRequest_(iTlsRequest *d, iSocket *sock, size_t num) {
+    d->totalBytesSent += num;
+    iNotifyAudienceArgs(d, sent, TlsRequestSent, d->totalBytesSent, d->totalBytesToSend);
+}
+
 static void setError_TlsRequest_(iTlsRequest *d, const char *msg) {
     setCStr_String(d->errorMsg, msg);
     setStatus_TlsRequest_(d, error_TlsRequestStatus);
@@ -817,6 +915,7 @@ void submit_TlsRequest(iTlsRequest *d) {
     clear_String(d->errorMsg);
     set_Block(&d->sending, &d->content);
     iRelease(d->socket);
+    d->certVerifyFailed = iFalse;
     SSL_set1_host(d->ssl, cstr_String(d->hostName));
     /* Server Name Indication for the handshake. */
     if (!contains_String(d->hostName, ':')) { /* Domain names only (not literal IPv6 addresses). */
@@ -831,6 +930,7 @@ void submit_TlsRequest(iTlsRequest *d) {
     iConnect(Socket, d->socket, connected, d, connected_TlsRequest_);
     iConnect(Socket, d->socket, disconnected, d, disconnected_TlsRequest_);
     iConnect(Socket, d->socket, readyRead, d, gotIncoming_TlsRequest_);
+    iConnect(Socket, d->socket, bytesWritten, d, bytesWritten_TlsRequest_);
     iConnect(Socket, d->socket, error, d, handleError_TlsRequest_);
     d->status = submitted_TlsRequestStatus;
     if (!open_Socket(d->socket)) {
@@ -872,6 +972,16 @@ const iString *errorMessage_TlsRequest(const iTlsRequest *d) {
     return d->errorMsg;
 }
 
+static void certificateVerifyFailed_TlsRequest_(iTlsRequest *d, const iTlsCertificate *cert) {
+    iAssert(d->cert == NULL);
+    d->cert = copy_TlsCertificate(cert);
+    d->certVerifyFailed = iTrue;
+}
+
+iBool isVerified_TlsRequest(const iTlsRequest *d) {
+    return !d->certVerifyFailed;
+}
+
 const iTlsCertificate *serverCertificate_TlsRequest(const iTlsRequest *d) {
     return d->cert;
 }
