@@ -32,6 +32,14 @@ iDeclareType(GmRequest)
 iDeclareType(Bookmark)
 iDeclareTypeConstruction(Bookmark)
 
+#define headings_BookmarkTag        "headings"
+#define homepage_BookmarkTag        "homepage"
+#define linkSplit_BookmarkTag       "linksplit"
+#define remote_BookmarkTag          "remote"
+#define remoteSource_BookmarkTag    "remotesource"
+#define subscribed_BookmarkTag      "subscribed"
+#define userIcon_BookmarkTag        "usericon"
+
 struct Impl_Bookmark {
     iHashNode node;
     iString url;
@@ -44,9 +52,23 @@ struct Impl_Bookmark {
 
 iLocalDef uint32_t  id_Bookmark (const iBookmark *d) { return d->node.key; }
 
-iBool   hasTag_Bookmark     (const iBookmark *d, const char *tag);
-void    addTag_Bookmark     (iBookmark *d, const char *tag);
-void    removeTag_Bookmark  (iBookmark *d, const char *tag);
+iBool   hasTag_Bookmark     (const iBookmark *, const char *tag);
+void    addTag_Bookmark     (iBookmark *, const char *tag);
+void    removeTag_Bookmark  (iBookmark *, const char *tag);
+
+iLocalDef void addTagIfMissing_Bookmark(iBookmark *d, const char *tag) {
+    if (!hasTag_Bookmark(d, tag)) {
+        addTag_Bookmark(d, tag);
+    }
+}
+iLocalDef void addOrRemoveTag_Bookmark(iBookmark *d, const char *tag, iBool add) {
+    if (add) {
+        addTagIfMissing_Bookmark(d, tag);
+    }
+    else {
+        removeTag_Bookmark(d, tag);
+    }
+}
 
 /*----------------------------------------------------------------------------------------------*/
 
