@@ -50,6 +50,8 @@ void    clear_Media             (iMedia *);
 iBool   setDownloadUrl_Media    (iMedia *, uint16_t linkId, const iString *url);
 iBool   setData_Media           (iMedia *, uint16_t linkId, const iString *mime, const iBlock *data, int flags);
 
+size_t  memorySize_Media        (const iMedia *);
+
 iMediaId        findLinkImage_Media (const iMedia *, uint16_t linkId);
 iBool           imageInfo_Media     (const iMedia *, iMediaId imageId, iGmMediaInfo *info_out);
 iInt2           imageSize_Media     (const iMedia *, iMediaId imageId);
@@ -59,6 +61,7 @@ size_t          numAudio_Media      (const iMedia *);
 iMediaId        findLinkAudio_Media (const iMedia *, uint16_t linkId);
 iBool           audioInfo_Media     (const iMedia *, iMediaId audioId, iGmMediaInfo *info_out);
 iPlayer *       audioPlayer_Media   (const iMedia *, iMediaId audioId);
+void            pauseAllPlayers_Media(const iMedia *, iBool setPaused);
 
 iMediaId        findLinkDownload_Media  (const iMedia *, uint16_t linkId);
 iBool           downloadInfo_Media      (const iMedia *, iMediaId downloadId, iGmMediaInfo *info_out);
