@@ -217,7 +217,7 @@ static void updateItems_SidebarWidget_(iSidebarWidget *d) {
     iBool isEmpty = iFalse; /* show blank? */
     switch (d->mode) {
         case feeds_SidebarMode: {
-            const iString *docUrl = withSpacesEncoded_String(url_DocumentWidget(document_App()));
+            const iString *docUrl = canonicalUrl_String(url_DocumentWidget(document_App()));
                                     /* TODO: internal URI normalization */
             iTime now;
             iDate on;
@@ -523,15 +523,17 @@ static void updateItems_SidebarWidget_(iSidebarWidget *d) {
             addChild_Widget(div, iClob(makePadding_Widget(gap_UI)));
             addChild_Widget(div, iClob(new_LabelWidget("${menu.identity.import}", "ident.import")));
             addChildFlags_Widget(div, iClob(new_Widget()), expand_WidgetFlag); /* pad */
+            iLabelWidget *linkLabel;
             setBackgroundColor_Widget(
                 addChildFlags_Widget(
                     div,
-                    iClob(new_LabelWidget(format_CStr(cstr_Lang("ident.gotohelp"),
+                    iClob(linkLabel = new_LabelWidget(format_CStr(cstr_Lang("ident.gotohelp"),
                                                       uiTextStrong_ColorEscape,
                                                       restore_ColorEscape),
                                           "!open newtab:1 gotoheading:1.6 url:about:help")),
-                    frameless_WidgetFlag | fixedHeight_WidgetFlag | wrapText_WidgetFlag),
+                    frameless_WidgetFlag | fixedHeight_WidgetFlag),
                 uiBackgroundSidebar_ColorId);
+            setWrap_LabelWidget(linkLabel, iTrue);
             addChild_Widget(d->blank, iClob(div));
         }
 //        arrange_Widget(d->blank);
@@ -616,7 +618,7 @@ static void updateMetrics_SidebarWidget_(iSidebarWidget *d) {
                 iMaxi(d->maxButtonLabelWidth,
                       3 * gap_UI + measure_Text(font_LabelWidget(d->modeButtons[i]),
                                                 translateCStr_Lang(normalModeLabels_[i]))
-                                       .x);
+                                       .bounds.size.x);
         }
     }
     updateItemHeight_SidebarWidget_(d);
@@ -625,13 +627,14 @@ static void updateMetrics_SidebarWidget_(iSidebarWidget *d) {
 void init_SidebarWidget(iSidebarWidget *d, enum iSidebarSide side) {
     iWidget *w = as_Widget(d);
     init_Widget(w);
-    setId_Widget(w, side == left_SideBarSide ? "sidebar" : "sidebar2");
+    setId_Widget(w, side == left_SidebarSide ? "sidebar" : "sidebar2");
     initCopy_String(&d->cmdPrefix, id_Widget(w));
     appendChar_String(&d->cmdPrefix, '.');
     setBackgroundColor_Widget(w, none_ColorId);
     setFlags_Widget(w,
                     collapse_WidgetFlag | hidden_WidgetFlag | arrangeHorizontal_WidgetFlag |
-                        resizeWidthOfChildren_WidgetFlag,
+                        resizeWidthOfChildren_WidgetFlag | noFadeBackground_WidgetFlag |
+                    noShadowBorder_WidgetFlag,
                     iTrue);
     iZap(d->modeScroll);
     d->side = side;
@@ -659,7 +662,7 @@ void init_SidebarWidget(iSidebarWidget *d, enum iSidebarSide side) {
     d->actions = NULL;
     /* On a phone, the right sidebar is used exclusively for Identities. */
     const iBool isPhone = deviceType_App() == phone_AppDeviceType;
-    if (!isPhone || d->side == left_SideBarSide) {
+    if (!isPhone || d->side == left_SidebarSide) {
         iWidget *buttons = new_Widget();
         setId_Widget(buttons, "buttons");
         for (int i = 0; i < max_SidebarMode; i++) {
@@ -678,8 +681,8 @@ void init_SidebarWidget(iSidebarWidget *d, enum iSidebarSide side) {
                              iClob(buttons),
                              arrangeHorizontal_WidgetFlag |
                                  resizeWidthOfChildren_WidgetFlag |
-                             arrangeHeight_WidgetFlag | resizeToParentWidth_WidgetFlag |
-                             drawBackgroundToHorizontalSafeArea_WidgetFlag);
+                             arrangeHeight_WidgetFlag | resizeToParentWidth_WidgetFlag); // |
+//                             drawBackgroundToHorizontalSafeArea_WidgetFlag);
         setBackgroundColor_Widget(buttons, uiBackgroundSidebar_ColorId);
     }
     else {
@@ -700,13 +703,13 @@ void init_SidebarWidget(iSidebarWidget *d, enum iSidebarSide side) {
     setPadding_Widget(as_Widget(d->list), 0, gap_UI, 0, gap_UI);
     addChildFlags_Widget(listAndActions,
                          iClob(d->list),
-                         expand_WidgetFlag | drawBackgroundToHorizontalSafeArea_WidgetFlag);
+                         expand_WidgetFlag); // | drawBackgroundToHorizontalSafeArea_WidgetFlag);
     setId_Widget(addChildPosFlags_Widget(listAndActions,
                                          iClob(d->actions = new_Widget()),
                                          isPhone ? front_WidgetAddPos : back_WidgetAddPos,
                                          arrangeHorizontal_WidgetFlag | arrangeHeight_WidgetFlag |
-                                             resizeWidthOfChildren_WidgetFlag |
-                                             drawBackgroundToHorizontalSafeArea_WidgetFlag),
+                                         resizeWidthOfChildren_WidgetFlag), // |
+//                                             drawBackgroundToHorizontalSafeArea_WidgetFlag),
                  "actions");
     setBackgroundColor_Widget(d->actions, uiBackgroundSidebar_ColorId);
     d->contextItem = NULL;
@@ -715,24 +718,24 @@ void init_SidebarWidget(iSidebarWidget *d, enum iSidebarSide side) {
     addChildFlags_Widget(content, iClob(d->blank), resizeChildren_WidgetFlag);
     addChildFlags_Widget(vdiv, iClob(content), expand_WidgetFlag);
     setMode_SidebarWidget(d,
-                          deviceType_App() == phone_AppDeviceType && d->side == right_SideBarSide ?
+                          deviceType_App() == phone_AppDeviceType && d->side == right_SidebarSide ?
                           identities_SidebarMode : bookmarks_SidebarMode);
     d->resizer =
         addChildFlags_Widget(w,
                              iClob(new_Widget()),
                              hover_WidgetFlag | commandOnClick_WidgetFlag | fixedWidth_WidgetFlag |
                                  resizeToParentHeight_WidgetFlag |
-                                 (side == left_SideBarSide ? moveToParentRightEdge_WidgetFlag
+                                 (side == left_SidebarSide ? moveToParentRightEdge_WidgetFlag
                                                            : moveToParentLeftEdge_WidgetFlag));
     if (deviceType_App() == phone_AppDeviceType) {
         setFlags_Widget(d->resizer, hidden_WidgetFlag | disabled_WidgetFlag, iTrue);
     }
-    setId_Widget(d->resizer, side == left_SideBarSide ? "sidebar.grab" : "sidebar2.grab");
+    setId_Widget(d->resizer, side == left_SidebarSide ? "sidebar.grab" : "sidebar2.grab");
     setBackgroundColor_Widget(d->resizer, none_ColorId);
     d->menu = NULL;
     addAction_Widget(w, SDLK_r, KMOD_PRIMARY | KMOD_SHIFT, "feeds.refresh");
     updateMetrics_SidebarWidget_(d);
-    if (side == left_SideBarSide) {
+    if (side == left_SidebarSide) {
         postCommand_App("~sidebar.update"); /* unread count */
     }
 }
@@ -773,6 +776,7 @@ static void itemClicked_SidebarWidget_(iSidebarWidget *d, iSidebarItem *item, si
             const iGmHeading *head = constAt_Array(headings_GmDocument(doc), item->id);
             postCommandf_App("document.goto loc:%p", head->text.start);
             dismissPortraitPhoneSidebars_Root(as_Widget(d)->root);
+            setOpenedFromSidebar_DocumentWidget(document_App(), iTrue);
             break;
         }
         case feeds_SidebarMode: {
@@ -783,7 +787,7 @@ static void itemClicked_SidebarWidget_(iSidebarWidget *d, iSidebarItem *item, si
         case bookmarks_SidebarMode:
         case history_SidebarMode: {
             if (!isEmpty_String(&item->url)) {
-                postCommandf_Root(get_Root(), "open newtab:%d url:%s",
+                postCommandf_Root(get_Root(), "open fromsidebar:1 newtab:%d url:%s",
                                  openTabMode_Sym(modState_Keys()),
                                  cstr_String(&item->url));
             }
@@ -799,7 +803,7 @@ static void itemClicked_SidebarWidget_(iSidebarWidget *d, iSidebarItem *item, si
                 updateContextMenu_SidebarWidget_(d);
                 arrange_Widget(d->menu);
                 openMenu_Widget(d->menu,
-                                d->side == left_SideBarSide
+                                d->side == left_SidebarSide
                                     ? topRight_Rect(itemRect_ListWidget(d->list, itemIndex))
                                     : addX_I2(topLeft_Rect(itemRect_ListWidget(d->list, itemIndex)),
                                               -width_Widget(d->menu)));
@@ -857,7 +861,7 @@ void setWidth_SidebarWidget(iSidebarWidget *d, float widthAsGaps) {
     if (!isFixedWidth) {
         /* Even less space if the other sidebar is visible, too. */
         const int otherWidth =
-            width_Widget(findWidget_App(d->side == left_SideBarSide ? "sidebar2" : "sidebar"));
+            width_Widget(findWidget_App(d->side == left_SidebarSide ? "sidebar2" : "sidebar"));
         width = iClamp(width, 30 * gap_UI, size_Root(w->root).x - 50 * gap_UI - otherWidth);
     }
     d->widthAsGaps = (float) width / (float) gap_UI;
@@ -938,33 +942,39 @@ static iBool handleSidebarCommand_SidebarWidget_(iSidebarWidget *d, const char *
         }
         const iBool isAnimated = prefs_App()->uiAnimations &&
                                  argLabel_Command(cmd, "noanim") == 0 &&
-                                 (deviceType_App() != phone_AppDeviceType);
+                                 (d->side == left_SidebarSide || deviceType_App() != phone_AppDeviceType);
         int visX = 0;
         if (isVisible_Widget(w)) {
             visX = left_Rect(bounds_Widget(w)) - left_Rect(w->root->widget->rect);
         }
         setFlags_Widget(w, hidden_WidgetFlag, isVisible_Widget(w));
+        /* Safe area inset for mobile. */
+        const int safePad = (d->side == left_SidebarSide ? left_Rect(safeRect_Root(w->root)) : 0);
         if (isVisible_Widget(w)) {
             setFlags_Widget(w, keepOnTop_WidgetFlag, iFalse);
             w->rect.size.x = d->widthAsGaps * gap_UI;
             invalidate_ListWidget(d->list);
             if (isAnimated) {
                 setFlags_Widget(w, horizontalOffset_WidgetFlag, iTrue);
-                setVisualOffset_Widget(w, (d->side == left_SideBarSide ? -1 : 1) * w->rect.size.x, 0, 0);
+                setVisualOffset_Widget(
+                    w, (d->side == left_SidebarSide ? -1 : 1) * (w->rect.size.x + safePad), 0, 0);
                 setVisualOffset_Widget(w, 0, 300, easeOut_AnimFlag | softer_AnimFlag);
             }
         }
         else if (isAnimated) {
             setFlags_Widget(w, horizontalOffset_WidgetFlag, iTrue);
-            if (d->side == right_SideBarSide) {
+            if (d->side == right_SidebarSide) {
                 setVisualOffset_Widget(w, visX, 0, 0);
-                setVisualOffset_Widget(w, visX + w->rect.size.x, 300, easeOut_AnimFlag | softer_AnimFlag);
+                setVisualOffset_Widget(
+                    w, visX + w->rect.size.x + safePad, 300, easeOut_AnimFlag | softer_AnimFlag);
             }
             else {
                 setFlags_Widget(w, keepOnTop_WidgetFlag, iTrue);
-                setVisualOffset_Widget(w, -w->rect.size.x, 300, easeOut_AnimFlag | softer_AnimFlag);
+                setVisualOffset_Widget(
+                    w, -w->rect.size.x - safePad, 300, easeOut_AnimFlag | softer_AnimFlag);
             }
         }
+        updateToolbarColors_Root(w->root);
         arrange_Widget(w->parent);
         /* BUG: Rearranging because the arrange above didn't fully resolve the height. */
         arrange_Widget(w);
@@ -984,11 +994,21 @@ static iBool processEvent_SidebarWidget_(iSidebarWidget *d, const SDL_Event *ev)
     /* Handle commands. */
     if (isResize_UserEvent(ev)) {
         checkModeButtonLayout_SidebarWidget_(d);
-        if (deviceType_App() == phone_AppDeviceType && d->side == left_SideBarSide) {
+        if (deviceType_App() == phone_AppDeviceType && d->side == left_SidebarSide) {
+            setFlags_Widget(w, rightEdgeDraggable_WidgetFlag, isPortrait_App());
             /* In landscape, visibility of the toolbar is controlled separately. */
             if (isVisible_Widget(w)) {
                 postCommand_Widget(w, "sidebar.toggle");
             }
+            setFlags_Widget(findChild_Widget(w, "buttons"),
+                            drawBackgroundToHorizontalSafeArea_WidgetFlag,
+                            isLandscape_App());
+            setFlags_Widget(findChild_Widget(w, "actions"),
+                            drawBackgroundToHorizontalSafeArea_WidgetFlag,
+                            isLandscape_App());
+            setFlags_Widget(as_Widget(d->list),
+                            drawBackgroundToHorizontalSafeArea_WidgetFlag,
+                            isLandscape_App());
             return iFalse;
         }
     }
@@ -1029,6 +1049,11 @@ static iBool processEvent_SidebarWidget_(iSidebarWidget *d, const SDL_Event *ev)
             postCommandf_App("sidebar.mode arg:%d toggle:1", identities_SidebarMode);
             return iTrue;
         }
+        else if (isPortraitPhone_App() && isVisible_Widget(w) && d->side == left_SidebarSide &&
+                 equal_Command(cmd, "swipe.forward")) {
+            postCommand_App("sidebar.toggle");
+            return iTrue;
+        }
         else if (startsWith_CStr(cmd, cstr_String(&d->cmdPrefix))) {
             if (handleSidebarCommand_SidebarWidget_(d, cmd + size_String(&d->cmdPrefix))) {
                 return iTrue;
@@ -1059,7 +1084,7 @@ static iBool processEvent_SidebarWidget_(iSidebarWidget *d, const SDL_Event *ev)
                 const int resMid = d->resizer->rect.size.x / 2;
                 setWidth_SidebarWidget(
                     d,
-                    ((d->side == left_SideBarSide
+                    ((d->side == left_SidebarSide
                          ? inner.x
                           : (right_Rect(rect_Root(w->root)) - coord_Command(cmd).x)) +
                      resMid) / (float) gap_UI);
@@ -1086,7 +1111,7 @@ static iBool processEvent_SidebarWidget_(iSidebarWidget *d, const SDL_Event *ev)
         else if (isCommand_Widget(w, ev, "bookmark.copy")) {
             const iSidebarItem *item = d->contextItem;
             if (d->mode == bookmarks_SidebarMode && item) {
-                SDL_SetClipboardText(cstr_String(withSpacesEncoded_String(&item->url)));
+                SDL_SetClipboardText(cstr_String(canonicalUrl_String(&item->url)));
             }
             return iTrue;
         }
@@ -1349,7 +1374,7 @@ static iBool processEvent_SidebarWidget_(iSidebarWidget *d, const SDL_Event *ev)
         else if (isCommand_Widget(w, ev, "history.copy")) {
             const iSidebarItem *item = d->contextItem;
             if (item && !isEmpty_String(&item->url)) {
-                SDL_SetClipboardText(cstr_String(withSpacesEncoded_String(&item->url)));
+                SDL_SetClipboardText(cstr_String(canonicalUrl_String(&item->url)));
             }
             return iTrue;
         }
@@ -1528,15 +1553,17 @@ static void draw_SidebarWidget_(const iSidebarWidget *d) {
     const iRect    bounds = bounds_Widget(w);
     iPaint p;
     init_Paint(&p);
-    if (flags_Widget(w) & visualOffset_WidgetFlag &&
-        flags_Widget(w) & horizontalOffset_WidgetFlag && isVisible_Widget(w)) {
-        fillRect_Paint(&p, boundsWithoutVisualOffset_Widget(w), tmBackground_ColorId);
+    if (deviceType_App() != phone_AppDeviceType) {
+        if (flags_Widget(w) & visualOffset_WidgetFlag &&
+            flags_Widget(w) & horizontalOffset_WidgetFlag && isVisible_Widget(w)) {
+            fillRect_Paint(&p, boundsWithoutVisualOffset_Widget(w), tmBackground_ColorId);
+        }
     }
     draw_Widget(w);
     if (isVisible_Widget(w)) {
         drawVLine_Paint(
             &p,
-            addX_I2(d->side == left_SideBarSide ? topRight_Rect(bounds) : topLeft_Rect(bounds), -1),
+            addX_I2(d->side == left_SidebarSide ? topRight_Rect(bounds) : topLeft_Rect(bounds), -1),
             height_Rect(bounds),
             uiSeparator_ColorId);
     }
@@ -1645,8 +1672,8 @@ static void draw_SidebarItem_(const iSidebarItem *d, iPaint *p, iRect itemRect,
             }
             /* Select the layout based on how the title fits. */
             int         metaFg    = isPressing ? fg : uiSubheading_ColorId;
-            iInt2       titleSize = advanceRange_Text(titleFont, range_String(&d->label));
-            const iInt2 metaSize  = advanceRange_Text(uiLabel_FontId, range_String(&d->meta));
+            iInt2       titleSize = measureRange_Text(titleFont, range_String(&d->label)).bounds.size;
+            const iInt2 metaSize  = measureRange_Text(uiLabel_FontId, range_String(&d->meta)).bounds.size;
             pos.x += iconPad;
             const int avail = width_Rect(itemRect) - iconPad - 3 * gap_UI;
             const int labelFg = isPressing ? fg : (isUnread ? uiTextStrong_ColorId : uiText_ColorId);
@@ -1655,7 +1682,7 @@ static void draw_SidebarItem_(const iSidebarItem *d, iPaint *p, iRect itemRect,
                 pos.y += (itemHeight - h2 - h2) / 2;
                 draw_Text(
                     uiLabel_FontId, addY_I2(pos, h2 - h1 - gap_UI / 8), metaFg, "%s \u2014 ", cstr_String(&d->meta));
-                int skip  = metaSize.x + advance_Text(uiLabel_FontId, " \u2014 ").x;
+                int skip  = metaSize.x + measure_Text(uiLabel_FontId, " \u2014 ").advance.x;
                 iInt2 cur = addX_I2(pos, skip);
                 const char *endPos;
                 tryAdvance_Text(titleFont, range_String(&d->label), avail - skip, &endPos);
@@ -1782,13 +1809,7 @@ static void draw_SidebarItem_(const iSidebarItem *d, iPaint *p, iRect itemRect,
                                                                      : uiTextFramelessHover_ColorId)
                                    : uiTextDim_ColorId;
         if (!d->listItem.isSelected && !isUsedOnDomain) {
-            /* Draw an outline of the icon. */
-            for (int off = 0; off < 4; ++off) {
-                drawRange_Text(font,
-                               add_I2(cPos, init_I2(off % 2 == 0 ? -1 : 1, off / 2 == 0 ? -1 : 1)),
-                               metaFg,
-                               range_String(&icon));
-            }
+            drawOutline_Text(font, cPos, metaFg, none_ColorId, range_String(&icon));
         }
         drawRange_Text(font,
                        cPos,
